#include <stdio.h>
#include <stdlib.h>

typedef unsigned int uint;

extern uint sat_unsigned_add(uint a, uint b);
extern uint sat_unsigned_sub(uint a, uint b);
extern uint sat_signed_add(uint a, uint b);
extern uint sat_signed_sub(uint a, uint b);

uint ref_sat_unsigned_add(uint a, uint b)
{
    long long r = (long long)a + (long long)b;
    if (r > 0xffffffffuLL)
      return 0xffffffff;
    return r;
}

uint ref_sat_unsigned_sub(uint a, uint b)
{
    if (a > b)
      return a-b;
    return 0;
}

uint ref_sat_signed_add(uint a, uint b)
{
    long long r = (long long)(int)a;
    r += (long long)(int)b;
    if (r > 0x7fffffffLL) return 0x7fffffff;
    if (r < -0x80000000LL) return 0x80000000;
    return r;
}

uint ref_sat_signed_sub(uint a, uint b)
{
    long long r = (long long)(int)a;
    r -= (long long)(int)b;
    if (r > 0x7fffffffLL) return 0x7fffffff;
    if (r < -0x80000000LL) return 0x80000000;
    return r;
}


void harness(uint a, uint b)
{
#define M(name) do { \
      uint r = name(a,b); \
      uint ex = ref_##name(a,b); \
      if (r != ex) { \
          printf(#name "(%#08x,%#08x) is %#08x, not %#08x\n", \
              a, b, r, ex); \
          exit(0); \
      } \
    } while (0)
    M(sat_unsigned_add);
    M(sat_unsigned_sub);
    M(sat_signed_add);
    M(sat_signed_sub);
#undef M
}

int main()
{
    int n[9] = {
        0x80000000,
        0x80000001,
        0xfffffffe,
        0xffffffff,
        0x00000000,
        0x00000001,
        0x00000002,
        0x7ffffffe,
        0x7fffffff
    };
    int i, j;
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
             harness(n[i], n[j]);
        }
    }
    for (i=0; i < 1000; ++i) {
        harness(rand(), rand());
    }
    return 0;
}
